@@ -55,6 +55,16 @@ void TestAuthSession::cleanupTestCase()
     qDebug() << "BYE!";
 }
 
+void TestAuthSession::sessionData()
+{
+    QVariantMap originalMap;
+    originalMap["Hello"] = "World";
+    originalMap["Int"] = 4;
+
+    SessionData sessionData(originalMap);
+    QCOMPARE(sessionData.toMap(), originalMap);
+}
+
 void TestAuthSession::queryMechanisms_existing_method()
 {
     AuthSession *as;
@@ -674,8 +684,6 @@ void TestAuthSession::cancel_with_delay()
 
     QObject::connect(as, SIGNAL(error(const SignOn::Error &)),
                      &loop, SLOT(quit()));
-    QObject::connect(as, SIGNAL(error(const SignOn::Error &)),
-                     &loop, SLOT(quit()),  Qt::DirectConnection);
     QTimer::singleShot(10*1000, &loop, SLOT(quit()));
 
     SessionData inData;
@@ -861,7 +869,7 @@ void TestAuthSession::processUi_with_existing_identity()
 
 
     SignOn::SessionData outData =
-        qVariantValue<SignOn::SessionData>(spy.at(0).at(0));
+        spy.at(0).at(0).value<SignOn::SessionData>();
     SsoTest2PluginNS::SsoTest2Data resultData =
         outData.data<SsoTest2PluginNS::SsoTest2Data>();
 
@@ -951,4 +959,6 @@ void TestAuthSession::windowId()
     QCOMPARE(errorCounter.count(), 0);
     QCOMPARE(m_signOnUI->clientData().value("WindowId").toUInt(),
              0xdeadbeef);
+    QCOMPARE(m_signOnUI->method(), QLatin1String("ssotest2"));
+    QCOMPARE(m_signOnUI->mechanism(), QLatin1String("mech1"));
 }
